@@ -7,6 +7,7 @@ use quote::{quote, quote_spanned, ToTokens};
 use syn::LitCStr;
 use syn::{ext::IdentExt, spanned::Spanned, Ident, Result};
 
+use crate::params::is_forwarded_args;
 #[cfg(feature = "experimental-inspect")]
 use crate::py_expr::PyExpr;
 use crate::pyfunction::{PyFunctionWarning, WarningFactory};
@@ -721,7 +722,9 @@ impl<'a> FnSpec<'a> {
                 // copy extracted arguments into async block
                 // output_py will create the owned arguments to store in the future
                 // output_args recreates the borrowed objects temporarily when building the future
-                let (output_py, output_args) = if !matches!(convention, CallingConvention::Noargs) {
+                let (output_py, output_args) = if !matches!(convention, CallingConvention::Noargs)
+                    && !is_forwarded_args(&self.signature)
+                {
                     (
                         Some(quote! {
                             let output = output.map(|o| o.map(Py::from));
@@ -733,6 +736,32 @@ impl<'a> FnSpec<'a> {
                 } else {
                     (None, None)
                 };
+                // if *args / **kwargs are present, treat the `Bound<'_, PyTuple>` / `Option<Bound<'_, PyDict>>` similarly
+                let (varargs_py, varargs_ptr) = if self.signature.python_signature.varargs.is_some()
+                {
+                    (
+                        Some(quote! {
+                            let _args = _args.to_owned().unbind();
+                        }),
+                        Some(quote! {
+                            let _args = _args.bind_borrowed(assume_attached.py());
+                        }),
+                    )
+                } else {
+                    (None, None)
+                };
+                let (kwargs_py, kwargs_ptr) = if self.signature.python_signature.kwargs.is_some() {
+                    (
+                        Some(quote! {
+                            let _kwargs = _kwargs.map(|k| k.to_owned().unbind());
+                        }),
+                        Some(quote! {
+                            let _kwargs = _kwargs.as_ref().map(|k| k.bind_borrowed(assume_attached.py()));
+                        }),
+                    )
+                } else {
+                    (None, None)
+                };
                 let args = self_arg.into_iter().chain(args);
                 let ok_wrap = quotes::ok_wrap(ret_ident.to_token_stream(), ctx);
                 quote! {
@@ -740,6 +769,8 @@ impl<'a> FnSpec<'a> {
                         let coroutine = {
                             #slf_py
                             #output_py
+                            #varargs_py
+                            #kwargs_py
                             #init_throw_callback
                             #pyo3_path::impl_::coroutine::new_coroutine(
                                 #pyo3_path::intern!(py, stringify!(#python_name)),
@@ -753,6 +784,8 @@ impl<'a> FnSpec<'a> {
                                         let py = assume_attached.py();
                                         #slf_ptr
                                         #output_args
+                                        #varargs_ptr
+                                        #kwargs_ptr
                                         function(#(#args),*)
                                     };
                                     let #ret_ident = future.await;
